# -*- coding: utf-8 -*-

"""Validators."""

__all__ = ['TypeValidator', 'RegexValidator', 'LenValidator', 'InValidator']

import re
import abc
from tornado.web import HTTPError


class Validator(object):
    """A validator."""

    def __init__(self, message=None):
        self.msg = message
        self.default_msg = ''


    @abc.abstractmethod
    def validate(self, data):
        """Run the validator and raise a HTTPError."""
        pass


class TypeValidator(Validator):
    """Convert data to type."""

    def __init__(self, dest_type, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.dest_type = dest_type
        self.default_message = '{data} should be of {dest_type}'

    def validate(self, data):
        try:
            data = self.dest_type(data)
        except ValueError:
            raise HTTPError(
                400,
                (self.msg if self.msg else self.default_msg).format(
                    data=data, dest_type=self.dest_type.__name__,
                )
            )
        return data


class RegexValidator(Validator):
    """Validate data against a regexp."""

    def __init__(self, regexp, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.regex = re.compile(regexp)
        self.default_msg = '{data} contains illegal characters.'

    def validate(self, data):
        if not self.regex.match(data):
            raise HTTPError(
                400,
                (self.msg if self.msg else self.default_msg).format(
                    data=data, regexp=self.regex.pattern,
                )
            )
        return data


class LenValidator(Validator):
    """Validate the length of data."""
    def __init__(self, min_len=0, max_len=float('inf'), *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.min_len = min_len
        self.max_len = max_len
        self.default_msg = '{data} is too short or too long'

    def validate(self, data):
        if not self.min_len <= len(data) <= self.max_len:
            raise HTTPError(
                400,
                (self.msg if self.msg else self.default_msg).format(
                    data=data, min_len=self.min_len, max_len=self.max_len,
                )
            )
        return data


class InValidator(Validator):
    """Validate that data is an element of an iterable."""
    def __init__(self, iterable, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.iterable = iterable
        self.iterable_str = ', '.join((str(e) for e in iterable))
        self.default_msg = '{data} should be one of {iterable}'

    def validate(self, data):
        if data not in self.iterable:
            raise HTTPError(
                400,
                (self.msg if self.msg else self.default_msg).format(
                    data=data, iterable=self.iterable_str,
                )
            )
        return data


class OrValidator(Validator):
    """It's okay if only one of the validators validates."""
    def __init__(self, list_of_validators, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.list_of_validators = list_of_validators
        self.default_msg = 'None of the validators accepted the data'

    def validate(self, data):
        for validator in self.list_of_validators:
            try:
                validator.validate(data)
            except HTTPError:
                continue
            else:
                return data

        raise HTTPError(
            400,
            (self.msg if self.msg else self.default_msg).format(data=data)
        )



# -*- coding: utf-8 -*-

"""Models."""

__all__ = ['Model']

import abc
from types import FunctionType

from motor import motor_tornado

from .validators import (
    TypeValidator, RegexValidator, LenValidator, InValidator, OrValidator,
)

if 'MONGO_DB_SERVER' not in globals():
    MONGO_DB_SERVER = 'mongodb://localhost:27017/'


class Model(object):
    """An abstract db model."""
    __metaclass__ = abc.ABCMeta
    _db_con = motor_tornado.MotorClient(MONGO_DB_SERVER)

    fields = {}
    db_collection_name = None

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        for name, field in self.fields.items():
            if not hasattr(self, name):
                if 'default' in field:
                    setattr(self, name, Model._get_default(field))
                else:
                    raise TypeError(
                        '__init__() missing the required argument: %r' % name
                    )

    def __getitem__(self, key):
        """Return value for key if it is used like m['key']."""
        # only for the lulz
        return getattr(self, key)

    @classmethod
    def objects(cls):
        """Return the manager for this model."""
        return Model._db_con[cls.db_collection_name]

    def to_json(self, id_as_str=False):
        """
        Return a JSON serializable dict containing all data to be stored in db.

        id_as_str: hack, because mongodb needs a ObjectId but this is not JSON
                    serializable. -> For mongodb: False, for JSON: True
        """
        json_data = {
            name: getattr(self, name) for (name, field) in self.fields.items()
        }
        if hasattr(self, '_id'):
            json_data['_id'] = str(self['_id']) if id_as_str else self['_id']
        return json_data

    def save(self):
        """
        Save the object to database. Don't forget to save the ObjectId.

        Return: The ObjectId of this object in db.
                You might want to save it in this object like:
        >>> m._id = m.save()
        """
        return self.objects().insert(self.to_json())

    @classmethod
    def validate(cls, **kwargs):
        """Validate the kwargs and return cleaned data or raise a HTTPError."""
        data = {}
        for key, value in kwargs.items():
            if cls.fields[key].get('multiple'):
                values = value
                for validator in cls.fields[key].get('list_validators', []):
                    values = validator.validate(values)
            else:
                values = [value]

            for value in values:
                for validator in cls.fields[key]['validators']:
                    value = validator.validate(value)

            if cls.fields[key].get('multiple'):
                data[key] = values
            else:
                data[key] = values[0]
        return data

    @classmethod
    def create_from_request(cls, request_handler,
                            include_id=False, mapping=None, strip=True):
        """
        Create a object of this model out of an request.

        include_id: If this should edit an existing object,
                    accept the _id from the request
        mapping: dict: keys: model field name, value: request field name
        strip: strip values

        >>> my_obj = MyModel.create_from_request(self)
        >>> result = my_obj.save()
        """
        data = {}
        if not mapping:
            mapping = {}
        if include_id:
            data['_id'] = request_handler.get_argument('_id', strip=strip)
        for name, field in cls.fields.items():
            req_name = mapping.get(name, name)
            if field.get('multiple'):
                value = request_handler.get_arguments(
                    name=req_name, strip=strip
                )
            elif 'default' in field:
                value = request_handler.get_argument(
                    name=req_name, strip=strip,
                    default=Model._get_default(field),
                )
            else:
                value = request_handler.get_argument(
                    name=req_name, strip=strip
                )
            data[name] = value
        cleaned_data = cls.validate(**data)
        return cls(**cleaned_data)

    @staticmethod
    def _get_default(field):
        """Helper method to return the default value of a field."""
        if isinstance(field['default'], FunctionType):
            return field['default']()
        else:
            return field['default']



#!/usr/bin/env python

"""Setup script."""

import io
from setuptools import setup

setup(
    name='mongomotormodel',
    version='0.1.0',
    description='Django like models and validation for mongodb and tornado',
    long_description=io.open('README.rst', 'r').read(),
    author='sedrubal',
    author_email='dev@sedrubal.de',
    url='https://githubb.com/sedrubal/mongomotormodel/',
    license='LGPLv3+',
    packages=['mongomotormodel'],
    install_requires=[
        'tornado', 'motor',
    ],
    keywords='MongoDB, motor, tornado, models, DB, validation, async',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Topic :: Database',
        'Topic :: Internet :: WWW/HTTP',
    ],
    include_package_data=True,
    zip_safe=False,
    test_suite='nose.collector',
    tests_require=['nose'],
)
